/**
 * easyWSDL - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.wsdl.impl.generic;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import javax.xml.transform.dom.DOMSource;

import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.XmlException;
import org.ow2.easywsdl.schema.api.absItf.AbsItfSchema;
import org.ow2.easywsdl.schema.util.SourceHelper;
import org.ow2.easywsdl.schema.util.StreamHelper;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.abstractElmt.AbstractWSDLReaderImpl;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription.WSDLVersionConstants;
import org.ow2.easywsdl.wsdl.util.Util;
import org.ow2.easywsdl.wsdl.util.WSDLVersionDetector;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class WSDLReaderImpl extends AbstractWSDLReaderImpl implements
org.ow2.easywsdl.wsdl.api.WSDLReader {

	List<Class> addedObjectFactories = null;

	private static Logger log = Logger
	.getLogger(WSDLReaderImpl.class.getName());

	private org.ow2.easywsdl.wsdl.api.WSDLReader reader11;
	private org.ow2.easywsdl.wsdl.api.WSDLReader reader20;
	
	public WSDLReaderImpl() throws WSDLException {
		reader11 = new org.ow2.easywsdl.wsdl.impl.wsdl11.WSDLReaderImpl();
		reader20 = new org.ow2.easywsdl.wsdl.impl.wsdl20.WSDLReaderImpl();
	}

	public WSDLReaderImpl(List<Class> addedObjectFactories) throws WSDLException {
		this.addedObjectFactories = addedObjectFactories;
		reader11 = new org.ow2.easywsdl.wsdl.impl.wsdl11.WSDLReaderImpl(
				this.addedObjectFactories);
		reader20 = new org.ow2.easywsdl.wsdl.impl.wsdl20.WSDLReaderImpl(
				this.addedObjectFactories);
	}

	public Description readWSDL(final URI wsdlURI) throws WSDLException {
		Description d = null;
		try {
			InputSource source1 = new InputSource(StreamHelper.getInputStream(wsdlURI));

			final WSDLVersionConstants version = WSDLVersionDetector
			.getVersion(source1);

			d = readWSDL(wsdlURI, version, new InputSource(StreamHelper.getInputStream(wsdlURI)), null, null);
		} catch (SchemaException e) {
			throw new WSDLException(e);
		}
		return d;
	}

	public Description readWSDL(final URI wsdlURI, Map<URI, AbsItfDescription> descriptionImports, Map<URI, AbsItfSchema> schemaImports) throws WSDLException {
		Description d = null;
		try {
		InputSource source1 = new InputSource(StreamHelper.getInputStream(wsdlURI));
		final WSDLVersionConstants version = WSDLVersionDetector
		.getVersion(source1);

		d = readWSDL(wsdlURI, version, new InputSource(StreamHelper.getInputStream(wsdlURI)), descriptionImports, schemaImports);
		} catch (SchemaException e) {
			throw new WSDLException(e);
		}
		return d;
	}

	public Description readWSDL(final URI wsdlURI, WSDLVersionConstants version, InputSource source) throws WSDLException {
		return readWSDL(wsdlURI, version, source, null, null);
	}		

	public Description readWSDL(final URI wsdlURI, WSDLVersionConstants version, InputSource source, Map<URI, AbsItfDescription> descriptionImports, Map<URI, AbsItfSchema> schemaImports) throws WSDLException {
		
		org.ow2.easywsdl.wsdl.api.WSDLReader reader = null;
		Description desc = null;
		try {
			if (version == WSDLVersionConstants.WSDL11) {
				reader = reader11;
				//reader = new org.ow2.easywsdl.wsdl.impl.wsdl11.WSDLReaderImpl(this.addedObjectFactories);
			} else if (version == WSDLVersionConstants.WSDL20) {
				reader = reader20;
				//reader = new org.ow2.easywsdl.wsdl.impl.wsdl20.WSDLReaderImpl(this.addedObjectFactories);
			} else {
				throw new WSDLException("unknown version of wsdl");
			}
			((AbstractWSDLReaderImpl) reader).setFeatures(this.getFeatures());
		} catch (final XmlException e) {
			throw new WSDLException("unknown version of wsdl", e);
		}

		desc = reader.readWSDL(wsdlURI, version, source, descriptionImports, schemaImports);

		return desc;
	}


	public Description readWSDL(Document doc) throws WSDLException {
		Description desc = null;
		try {
			URI uri = null;
			if(doc.getDocumentURI() != null) {
				uri = new URI(doc.getDocumentURI());
			} else {
				uri = new URI(".");
			}

			WSDLVersionConstants version = Util.getVersionInDocument(doc);

			desc = this.readWSDL(uri, version, SourceHelper.convertDOMSource2InputSource(new DOMSource(doc)));
		} catch (URISyntaxException e) {
			throw new WSDLException(e);
		} catch (XmlException e) {
			throw new WSDLException(e);
		} 
		return desc;
	}

	public Description readWSDL(Document doc,
			Map<URI, AbsItfDescription> descriptionImports,
			Map<URI, AbsItfSchema> schemaImports) throws WSDLException {
		Description desc = null;
		try {
			URI uri = null;
			if(doc.getDocumentURI() != null) {
				uri = new URI(doc.getDocumentURI());
			} else {
				uri = new URI(".");
			}

			WSDLVersionConstants version =  Util.getVersionInDocument(doc);

			desc = this.readWSDL(uri, version, SourceHelper.convertDOMSource2InputSource(new DOMSource(doc)), descriptionImports, schemaImports);
		} catch (URISyntaxException e) {
			throw new WSDLException(e);
		} catch (XmlException e) {
			throw new WSDLException(e);
		}
		return desc;
	}
}
