/**
 * easyWSDL - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.wsdl.impl.wsdl11;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.sax.SAXSource;

import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.SchemaReader;
import org.ow2.easywsdl.schema.api.XmlException;
import org.ow2.easywsdl.schema.api.absItf.AbsItfSchema;
import org.ow2.easywsdl.schema.impl.SchemaReaderImpl;
import org.ow2.easywsdl.schema.util.EasyXMLFilter;
import org.ow2.easywsdl.schema.util.SourceHelper;
import org.ow2.easywsdl.schema.util.StreamHelper;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLImportException;
import org.ow2.easywsdl.wsdl.api.abstractElmt.AbstractWSDLReaderImpl;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription.WSDLVersionConstants;
import org.ow2.easywsdl.wsdl.org.xmlsoap.schemas.wsdl.TDefinitions;
import org.ow2.easywsdl.wsdl.util.Util;
import org.ow2.easywsdl.wsdl.util.WSDLVersionDetector;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class WSDLReaderImpl extends AbstractWSDLReaderImpl implements org.ow2.easywsdl.wsdl.api.WSDLReader {

	private static Logger log = Logger.getLogger(WSDLReaderImpl.class.getName());

	
	private WSDLJAXBContext jaxbcontext = null;
	

	/*
	 * Private object initializations
	 */
	public WSDLReaderImpl() throws WSDLException {
		super();
		try {
			this.jaxbcontext = new WSDLJAXBContext();
		} catch (final SchemaException e) {
			throw new WSDLException(e);
		}
	}

	public WSDLReaderImpl(List<Class> addedObjectFactories) throws WSDLException {
		super();
		try {
			this.jaxbcontext = new WSDLJAXBContext(addedObjectFactories);
		} catch (final WSDLException e) {
			throw new WSDLException(e);
		}
	}

	/**
	 * @return the jaxbContext
	 */
	public JAXBContext getJaxbContext() {
		return this.jaxbcontext.getJaxbContext();
	}

	public Description readWSDL(final URI wsdlURI) throws WSDLException {
		Description d = null;
		try {
			InputSource source1 = new InputSource(StreamHelper.getInputStream(wsdlURI));
			final WSDLVersionConstants version = WSDLVersionDetector.getVersion(source1);

			d = readWSDL(wsdlURI, version, new InputSource(StreamHelper.getInputStream(wsdlURI)), null, null);
		} catch (SchemaException e) {
			throw new WSDLException(e);
		}
		return d;
	}

	public Description readWSDL(final URI wsdlURI, Map<URI, AbsItfDescription> descriptionImports, Map<URI, AbsItfSchema> schemaImports) throws WSDLException {
		return this.readWSDL(wsdlURI, descriptionImports, schemaImports, true);
	}
	
	public Description readWSDL(final URI wsdlURI, Map<URI, AbsItfDescription> descriptionImports, Map<URI, AbsItfSchema> schemaImports, boolean deleteImports) throws WSDLException {
		Description d = null;
		try {
			InputSource source1 = new InputSource(StreamHelper.getInputStream(wsdlURI));
			final WSDLVersionConstants version = WSDLVersionDetector.getVersion(source1);

			//d = readWSDL(wsdlURI, version, source1, descriptionImports, schemaImports);
			d = readWSDL(wsdlURI, new InputSource(StreamHelper.getInputStream(wsdlURI)), descriptionImports, schemaImports, deleteImports);
		} catch (SchemaException e) {
			throw new WSDLException(e);
		}
		return d;
	}

	public Description readWSDL(final URI wsdlURI, WSDLVersionConstants version, InputSource source) throws WSDLException {
		return readWSDL(wsdlURI, version, source, null, null);
	}

	public Description readWSDL(final URI wsdlURI, WSDLVersionConstants version, InputSource source, Map<URI, AbsItfDescription> descriptionImports, Map<URI, AbsItfSchema> schemaImports) throws WSDLException {
		return readWSDL(wsdlURI, source, descriptionImports, schemaImports);
	}
	
	public Description readWSDL(final URI wsdlURI, InputSource source, Map<URI, AbsItfDescription> descriptionImports, Map<URI, AbsItfSchema> schemaImports) throws WSDLException {
		return this.readWSDL(wsdlURI, source, descriptionImports, schemaImports, true);
	}
	
	private Description readWSDL(final URI wsdlURI, InputSource source, Map<URI, AbsItfDescription> descriptionImports, Map<URI, AbsItfSchema> schemaImports, boolean deleteImports) throws WSDLException {
		Description desc = null;
		if(deleteImports){
			this.getImportList().clear();
		}
		try {
			
			log.fine("Loading " + wsdlURI);
			
			XMLReader xmlReader = XMLReaderFactory.createXMLReader();
			EasyXMLFilter filter = new EasyXMLFilter(xmlReader);
			SAXSource saxSource = new SAXSource(filter, source);
			
			// TODO use SAX validation instead of JAXB validation
			// turn off the JAXB provider's default validation mechanism to
			// avoid duplicate validation
			// SchemaReaderImpl.getUnMarshaller().setValidating( false );
			
			Unmarshaller unmarshaller = this.jaxbcontext.getJaxbContext().createUnmarshaller();
			final JAXBElement<TDefinitions> wsdlBinding = unmarshaller.unmarshal(saxSource, TDefinitions.class);
			TDefinitions def = wsdlBinding.getValue();

			desc = new org.ow2.easywsdl.wsdl.impl.wsdl11.DescriptionImpl(wsdlURI, def, filter.getNamespaceMapper(), filter.getSchemaLocator(), this.getFeatures(), descriptionImports, schemaImports, this);

		} catch (JAXBException e) {
			throw new WSDLException("Can not get wsdl at: " + wsdlURI, e);
		} catch (SAXException e) {
			throw new WSDLException("Can not get wsdl at: " + wsdlURI, e);
		} catch (WSDLImportException e) {
			throw new WSDLException("Can not get wsdl at: " + wsdlURI, e);
		} catch (NumberFormatException e) {
			throw new WSDLException("Can not get wsdl at: " + wsdlURI + " WSDL too large !", e);
		}

		return desc;
	}

	public Description readWSDL(Document doc) throws WSDLException {
		Description desc = null;
		try {
			URI uri = null;
			if (doc.getDocumentURI() != null) {
				uri = new URI(doc.getDocumentURI());
			} else {
				uri = new URI(".");
			}

			WSDLVersionConstants version = Util.getVersionInDocument(doc);

			desc = this.readWSDL(uri, version, SourceHelper.convertDOMSource2InputSource(new DOMSource(doc)));
		} catch (URISyntaxException e) {
			throw new WSDLException(e);
		} catch (XmlException e) {
			throw new WSDLException(e);
		}
		return desc;
	}

	public Description readWSDL(Document doc, Map<URI, AbsItfDescription> descriptionImports, Map<URI, AbsItfSchema> schemaImports) throws WSDLException {
		Description desc = null;
		try {
			URI uri = null;
			if (doc.getBaseURI() != null) {
				uri = new URI(doc.getBaseURI());
			} else {
				uri = new URI(".");
			}

			WSDLVersionConstants version = Util.getVersionInDocument(doc);

			desc = this.readWSDL(uri, version, SourceHelper.convertDOMSource2InputSource(new DOMSource(doc)), descriptionImports, schemaImports);
		} catch (URISyntaxException e) {
			throw new WSDLException(e);
		} catch (XmlException e) {
			throw new WSDLException(e);
		}
		return desc;
	}

}
